#include "sut.h"
/*D
   pexec - Executes a program on a specified set of nodes

   Arguments:
.  range - A range of nodes.  Must be the first argument.  See below
.  args  - Program to run, and its command-line arguments.

   pexec also reads a number of environment variables.  These are
.  SUT_REGULATE - 

   In addition, there are some environment variables that are used for 
   debugging.   These are
.  SUT_PRINT  - Prints out the actions that pexec is taking.
.  SUT_HANDSHAKE - Prints out actions during the handshake at remote
   process startup.
.  SUT_PASSDATA - Prints outs actions during the handling of output from
   the user-specified program
.  SUT_FORK   - Do not actually perform fork call.  Used only for
   debugging; pexec will not operate correctly if this is set.
.  SUT_NOEXEC - Do not actually perform execvp call.  Used only for
   debugging; pexec will not operate correctly if this is set.

   Notes:
   The range of nodes is specified as a comma-separated series of ranges.
   A range is specified as n-m, where n is the first process and m is the
   last.  As a special case, the expression '-all' selects all nodes.

   Nodes are numbered from 1.

   Examples:
$
$  pexec -all hostname
$        Give the names of all of the nodes
$  pexec -all df /tmp | grep -v Filesystem
$        List the df statistics of the local disks
$  pexec -all ps aux | grep -v root
$        List all of the non-root processes
$  pexec 3 ps aux
$        List all processes on node 3 (similar to rsh <name of 3> ps aux)
$
   See also:
   phostname - converts leading number to name of node
D*/
main(argc, argv)
int argc;
char **argv;
{
SUT_Jobs    jobs;
char        *(*HostnumToName)();
void        *HNctx;
char        *(*ExeNameF)();
void        *ExeCtx;
char        **(*ArgsF)();
void        *ArgsC();
char        *Environment;
extern char *SUT_HostName();
int         err;
char        prun_path[1024];
int         *host_nums, count;


#ifdef SUT_SLAVE
/* If we are the slave, then the first action is to perform the handshake */
if (SUT_Debug & SUT_HANDSHAKE)
    fprintf( stderr, "Starting handshake...\n" );
/* Note that the handshake gets the debug and option values */
SUT_Handshake( 0, 1, 0 );
#else
DEBUG(printf("Getting debug options...\n" );)
SUT_Get_debug();
DEBUG(printf("Debug value is %d\n", SUT_Debug );)
SUT_Get_options();
#endif

/* Need to get initial range from command line */
if (err = SUT_Get_node_list( argc, argv, &count, &host_nums, &HNctx )) {
    fprintf( stderr, "Could not get process list from command line\n" );
    SUT_Errormessage( stderr, err );
    exit(1);
    }

#ifndef SUT_SLAVE
/* The slaves always get a 'rotated' host list */
if (err = SUT_Rotate_host_list( host_nums, count )) {
    SUT_Errormessage( stderr, err );
    exit(1);
    }
#endif

/* This will handle any SIGCHLDs (made global so that there are 
   no race conditions from one call to the next) 
 */
SUT_Init( &jobs );
#ifndef SUT_SLAVE
jobs.writeparent = SUT_Writedataroot;
#endif
SUT_Set_jobs_data( &jobs );

/* If we aren't STARTING on the local node, we need to move there before
   continuing.  
 */
#ifdef SUT_TESTING
sprintf( prun_path, "%s/%s", "/home/gropp/sut/src", "pexec.slave" );
#else
sprintf( prun_path, "%s/%s", SUTEXEDIR, "pexec.slave" );
#endif
#ifndef SUT_SLAVE
if (SUT_Notlocal( host_nums, count, SUT_HostName, HNctx,
		  prun_path, argv, &jobs ) == SUT_NOT_LOCAL) {
    if (err = SUT_Passdata( &jobs )) {
	fprintf( stderr, "Error in managing data\n" );
	SUT_Errormessage( stderr, err );
	exit(1);
	}
    return 0;
    }
#endif

Environment = 0;
ExeNameF    = 0;
/* Eventually, this will take a directory from the Makefile/configure */
ExeCtx	    = (void *)prun_path;
if (err = SUT_Pstart( host_nums, count, SUT_HostName, HNctx, 
		      ExeNameF, ExeCtx, (char **(*)())0, (void *)(argv+2), 
		      Environment, &jobs )) {
    fprintf( stderr, "Error in starting jobs\n" );
    SUT_Errormessage( stderr, err );
    exit(1);
    }

/* Here is where we should run the local program (argv[2] on) in a 
   subprocess, adding its stdin/stdout/stderr to the "jobs" structure 
 */
if (err = SUT_Startprocess( &jobs, argv+2 )) {
    fprintf( stderr, "Error in starting program %s\n", argv[2] );
    SUT_Errormessage( stderr, err );
    exit(1);
    }

if (SUT_Options & SUT_REGULATE) 
    err = SUT_Regulate_data( &jobs );
else
    err = SUT_Passdata( &jobs ); 
if (err) {
    fprintf( stderr, "Error in managing data\n" );
    SUT_Errormessage( stderr, err );
    exit(1);
    }

if (SUT_Debug & SUT_PRINT) {
    fprintf( stderr, "[%d] pexec exiting\n", SUT_Mynode );
    }
#ifdef SUT_SLAVE
SUT_Exit( &jobs );
/* Force a close of the output */
close(1);
#else
#endif

/* Discontinue handling of signals */
SUT_No_sigchld( );

return jobs.retstat;
}
